/*//////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  Stupid Clustering Algorithm 
//
//  Copyright 1994-1999, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//      Ionel Daniel Stroe 
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~xmdv
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  XmdvTool Development Team: <matt@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT */

#include	<stdio.h>
#include	<math.h>
#include	<string.h>
#include	"c.h"

int		nodeID[M];
int		numberOfPoints[M];
int		cc[M];
int		nextID = M;
double		radius[M];
double		node[M][N];

void readDataPoints()
{
  int		i, j;

  for(j=0; j<M; j++)
  for(i=0; i<N; i++)
  scanf("%lf", &node[j][i]);
}

/*							    sum		*/
void writeCluster(int id, int pid, int m)
{
  int		i;

  printf("%10d%10d%10d", id, pid, numberOfPoints[m]);
  for(i=0; i<N; i++) printf("%15.6lf", node[m][i] * numberOfPoints[m]);
  printf("%10.6lf\n", radius[m]);
}

/*							average		*/
/*
void writeCluster(int id, int pid, int m)
{
  int		i;

  printf("%10d%10d%10d", id, pid, numberOfPoints[m]);
  for(i=0; i<N; i++) printf("%20.9lf", node[m][i]);
  printf("%10.6lf\n", radius[m]);
}
*/

double distance(int m1, int m2)
{
  int		i;
  double	s = 0.;

  for(i=0; i<N; i++) s += (node[m1][i] - node[m2][i])*
		       (node[m1][i] - node[m2][i]);
  return sqrt(s);
}

void changeConnexComponent(int noc, int v1, int v2)
{
  int		i;

  for(i=0; i<noc; i++) if(cc[i]==v1) cc[i]=v2;
}

getConnexComponents(int noc, double r)
{
  int		i, j;
  int		lastTag = 0;

  memset(cc, 0, sizeof(cc));

  for(i=0; i<noc; i++)
  for(j=i+1; j<noc; j++)
  if(distance(i, j) <= r)
  if(!cc[i] && !cc[j]) cc[i] = cc[j] = ++lastTag;
  else if(!cc[i]) cc[i] = cc[j];
  else if(!cc[j]) cc[j] = cc[i];
  else if(cc[i] && cc[j]) changeConnexComponent(noc, cc[i], cc[j]);

  return lastTag;
}

void addNode(int m1, int m2)
{
  int		i;

  for(i=0; i<N; i++) node[m1][i] = (numberOfPoints[m1]*node[m1][i] +
      numberOfPoints[m2]*node[m2][i])/(numberOfPoints[m1]+numberOfPoints[m2]);
  numberOfPoints[m1] += numberOfPoints[m2];
  numberOfPoints[m2] = 0;
}

void aggregateTag(int tag, int noc, double r)
{
  int		first = 0, i;

  while(first<noc && cc[first]!=tag) first++;
  if(first==noc) return;

  writeCluster(nodeID[first], nextID, first);

  radius[first] = r;
  nodeID[first] = nextID;

  for(i=first+1; i<noc; i++)
  if(cc[i]==tag)
  {
	writeCluster(nodeID[i], nextID, i);
	addNode(first, i);
	numberOfPoints[i] = 0;
  }

  nextID++;
}

void copyNode(int m1, int m2)
{
  int		i;

  nodeID[m1] = nodeID[m2];
  numberOfPoints[m1] = numberOfPoints[m2]; 
  numberOfPoints[m2] = 0; 
  radius[m1] = radius[m2];
  for(i=0; i<N; i++) node[m1][i] = node[m2][i];
}

compact(int noc)
{
  int		i, next;

  for(i=0; ;i++)
  {
	while(numberOfPoints[i]) i++;
	next = i+1;
	while(next<noc && !numberOfPoints[next]) next++;
	if(next<noc) copyNode(i, next);
	else return i;
  }
}

void main()
{
  double	Radius = 0.;
  int		numberOfClusters = M;
  int		numberOfConnComp = M;
  int		i;
 
  readDataPoints();
  for(i=0; i<M; i++)
  {
	nodeID[i] = i;
	numberOfPoints[i] = 1;
  }

  for(; numberOfClusters > 1; Radius += dR)
  {
	fprintf(stderr, "RAD: %f\n", Radius);
	numberOfConnComp = getConnexComponents(numberOfClusters, Radius);
	if(DEBUG) fprintf(stderr, "#CC: %d\n", numberOfConnComp);
	for(i=1; i<=numberOfConnComp; i++)
	{
		aggregateTag(i, numberOfClusters, Radius);
	}
	numberOfClusters = compact(numberOfClusters);
	if(DEBUG) fprintf(stderr, "#CL: %d\n", numberOfClusters);
  }

  writeCluster(nextID-1, -1, 0);
  puts("");		/* important for m.c */
}

